@@ -1,7 +1,11 @@
 ## ---- include = FALSE---------------------------------------------------------
 knitr::opts_chunk$set(
   collapse = TRUE,
-  #dev="png",
+  #dev="svg",
+  dpi=50,
+  fig.width=7, fig.height=5.5,
+  out.width="600px",
+  fig.retina=1,
   comment = "#>"
 )
 fullVignette <- Sys.getenv("_R_FULL_VIGNETTE_") %in% c("1","TRUE")
